<?php 

/**
 * @file
 * Configurations options for the GeoBlocker module.
 */

/**
 * Implementation of hook_form()
 */
function geoblocker_admin_settings() {
  $form = array();

  drupal_set_message(t('Please make sure any WYSIWYG editor like CKEditor or TinyMCE is disabled on this page.'), 'error');
  
  drupal_add_js(drupal_get_path('module', 'geoblocker') .'/geoblocker.js');
  
  $defaults = array(
    'mode' => 'black',
    'countries' => '',
    'scope' => 'complete',
    'scope_nodes' => '',
    'whitelisted_ipv4s' => geoblocker_admin_default_whitelisted_ips('ipv4'),
    'cron_ttl' => 60*60*24*30,
    'default_action' => 0,
    'db_status' => 0,
  );
  $vars = variable_get('geoblocker', $defaults);
  if (count($vars) == 1) {
    // only db_status is there, see geoblocker.install
    $vars += $defaults;
  }
  
  $form['geoblocker_db_status'] = array(
    '#type' => 'hidden',
    '#value' => $vars['db_status'],
  );
  
  $msg  = 'Determines how to process incoming requests. ';
  $msg .= '"Black listing" disallows access for visitors coming from configured countries. ';
  $msg .= '"White listing" allows access only for visitors coming from configured countires.';
  $form['geoblocker']['mode'] = array(
    '#type' => 'select',
    '#parents' => array('geoblocker', 'mode'),
    '#title' => t('Access Control Method'),
    '#options' => array('white' => t('White listing'), 'black' => t('Black listing')),
    '#default_value' => $vars['mode'],  
    '#description' => t($msg),
  );  
  
  $msg  = 'A comma separated list of 3-letter country codes in conformance to the ISO 3166 alpha 3 standard. ';
  $msg .= 'See !url for more information.';
  $form['geoblocker']['countries'] = array(
    '#type' => 'textarea',
    '#parents' => array('geoblocker', 'countries'),
    '#title' => t('Countries'),
    '#default_value' => $vars['countries'],
    '#description' => t($msg, array('!url' => l('ISO 3166', 'http://en.wikipedia.org/wiki/ISO_3166-1_alpha-3'))),
  );
  
  $msg  = 'The scope setting allows to configure the scope of the GeoBlocker module. ';
  $msg .= '"Complete Web Site" restricts/allows access to the complete Web Site. ';
  $msg .= '"A Subset of the Web Site" restricts/allows access only to a specific subset of nodes.';
  $form['geoblocker']['scope'] = array(
    '#type' => 'select',
    '#parents' => array('geoblocker', 'scope'),
    '#title' => t('Access Control Scope'),
    '#options' => array('complete' => t('Complete Web Site'), 'subset' => t('A Subset of the Web Site')),
    '#default_value' => $vars['scope'],  
    '#description' => t($msg),
  );

  $msg  = 'A comma separated list of node IDs to apply the GeoBlocker module to. !br';
  $msg .= 'This setting is ignored if the selected scope is "Complete Web Site".';
  $form['geoblocker']['scope_nodes'] = array(
    '#type' => 'textarea',
    '#parents' => array('geoblocker', 'scope_nodes'),
    '#title' => t('Nodes belonging to the Scope "Subset"'),
    '#default_value' => $vars['scope_nodes'],
    '#description' => t($msg, array('!br' => '<br/>')),  
  );
  
  $msg  = 'By default loopback requests, as well some other !reserved IPv4 ranges are allowed. ';
  $msg .= 'In this field you can put additional IPv4 ranges to be whitelisted to have access to the site. !br';
  $msg .= 'Due to performance considerations IPv4 addresses must be entered in unsigned !long representation format. !br';
  $msg .= 'You could use !this JS tool to convert IPv4 dotted notation into IPv4 Decimal notation.!br';
  $msg .= 'No wildcards, no regexp\'s, only complete IPv4 IPs.';
  $whitelisted_ipv4_ranges = $vars['whitelisted_ipv4s'];
  $whitelisted_ipv4_form_string = '';
  $msg_whitelisted = '';
  foreach ($whitelisted_ipv4_ranges as $entry) {
    if ($entry[0] == $entry[1]) {
      $whitelisted_ipv4_form_string .= $entry[0] ."\r\n";
      $msg_whitelisted .= $entry[0] .' = '. long2ip($entry[0]);  
    }
    else {
      $whitelisted_ipv4_form_string .= $entry[0] .'|'. $entry[1] ."\r\n";
      $msg_whitelisted .= $entry[0] .'|'. $entry[1] .' = '. long2ip($entry[0]) .'|'. long2ip($entry[1]);
    }
  }
  $msg .= '!br'. $msg_whitelisted;
  $form['geoblocker']['whitelisted_ipv4s'] = array(
    '#type' => 'textarea',
    '#parents' => array('geoblocker', 'whitelisted_ipv4s'),
    '#title' => t('Whitelisted IPv4 ranges'),
    '#default_value' => $whitelisted_ipv4_form_string,
    '#description' => t($msg, 
      array(
        '!reserved' => l(t('reserved'), 'http://en.wikipedia.org/wiki/Reserved_IP_addresses#Reserved_IPv4_addresses'),      
        '!br' => '<br/>', 
        '!long' => l(t('long'), 'http://php.net/ip2long'),
        '!this' => '<a href="#"><span id="geoblocker-ipv42long">'. t('this') .'</span></a>',
      )
    ),  
  );
  
  $msg  = 'How often sould the system update the DB with country specific ranges. ';
  $msg .= 'Enter the time perdiod in seconds. ';
  $msg .= 'Time periods shorter than 30 are rather pointless since the growth rate of ';
  $msg .= 'IPv4 ranges is rather slow and changes of IPv4 ranges assigned to countries are very rare and uncommon.';
  $form['geoblocker']['cron_ttl'] = array(
    '#type' => 'textfield',
    '#parents' => array('geoblocker', 'cron_ttl'),
    '#title' => t('DB Refresh Time'),
    '#default_value' => $vars['cron_ttl'], // 30 days
    '#description' => t($msg)
  );
  
  $msg  = 'Default action if the system can not determine the country the visitor originates from. ';
  $msg .= 'This setting is independant from the mode setting and is invoked only if the system is unable ';
  $msg .= 'to determine the country of origin of the visitor.'; 
  $form['geoblocker']['default_action'] = array(
    '#type' => 'select',
    '#parents' => array('geoblocker', 'default_action'),
    '#title' => t('Default action'),
    '#options' => array(0 => 'Block', 1 => 'Allow'),
    '#default_value' => $vars['default_action'],
    '#description' => t($msg),
  );
  
  $form['#validate'] = array('geoblocker_admin_settings_validate');
  
  return system_settings_form($form);
}

/**
 * Implementation of hook_validate()
 */
function geoblocker_admin_settings_validate($form, &$form_state) {
  $form_state['values']['geoblocker']['db_status'] = $form_state['values']['geoblocker_db_status'];
  unset($form_state['values']['geoblocker_db_status']);
  
  $whitelisted_ipv4_ranges = $form_state['values']['geoblocker']['whitelisted_ipv4s'];
  $whitelisted_ipv4_ranges = explode("\r\n", trim($whitelisted_ipv4_ranges));

  $new_whitelisted_ipv4_ranges = array();
  foreach ($whitelisted_ipv4_ranges as $entry) {
    $entry = trim($entry);
    if (strpos($entry, '|') === FALSE) {
      $entry = sprintf("%u", $entry);
      $new_whitelisted_ipv4_ranges[] = array($entry, $entry);    
    }
    else {
      $entry = explode('|', $entry);
      $new_whitelisted_ipv4_ranges[] = array(sprintf("%u", $entry[0]), sprintf("%u", $entry[1]));
    }
  }
  $form_state['values']['geoblocker']['whitelisted_ipv4s'] = $new_whitelisted_ipv4_ranges;
}

/**
 * default whitelisted IPs
 * keep this func in sync with geoblocker.module/function geoblocker_default_whitelisted_ips()
 * @param string $type  IPv4 vs IPv6
 */
function geoblocker_admin_default_whitelisted_ips($type = 'ipv4') {
  $ips = array();
  
  switch ($type) {
    case 'ipv4': 
      $ips = array(array('2130706432', '2147483646')); 
      break;
    case 'ipv6':
      $ips = array();
      break;
  }
  
  return $ips;
}